# Django backend ONE.UP

[![coverage report](https://gitup.uni-potsdam.de/innohs/rest-api/badges/master/coverage.svg)](https://gitup.uni-potsdam.de/innohs/rest-api/commits/master)

## Table of Contents
- [The Project](#the-project)
- [How to run](#how-to-run)
- [Developed Apps](#developed-apps)
- [Custom Admin Functions](#custom-admin-functions)
- [Steps to clean data](#steps-to-clean-data)

## The Project

This Django based backend was developed as part of the
[Innovative Hochschule Potsdam - ONE.UP](https://www.uni-potsdam.de/en/multimedia/projects/innovative-hochschule)
project.

The aim of this project was to empower scientific communication. Many scientists learn skills to communicate within
their scientific community, but thereby unlearn the language and platforms to reach audiences outside their community.
If scientists are willing to open their communication spheres, then finding a medium and the appropriate language should
be, at best, as little effort as possible. Our approach is to scan the media landscape for the chosen research topics
and prepare them for researchers in a way that they know which of their research topics are currently being discussed by
society. With this knowledge, researchers can find media with specific thematic focuses and use this for targeted and
appropriate science communication. The underlying assumption is that scientists might get an easier entry point for the
communication about their research process or results when they know about the pervasion of their research interests in
society. [From the abstract for the presentation a ICDT '22](https://lnu.se/contentassets/cfe41f612c2a47328fc4c6f396106a48/program_isdt_symposium_2022.pdf)

A paper about possible features for this architecture can be found
at [EUNIS '20](https://www.eunis.org/download/2020/EUNIS_2020_paper_67.pdf).

This project is divided into two main parts: Backend (this repository)
and [Dashboard](https://gitup.uni-potsdam.de/innohs/dashboard).

The main tasks of the backend are:

* persist data
* create an API to show the aggregated data in the dashboard
* read out newspaper articles
* learn classifier to label newspaper articles
* read out [news from University of Potsdam](https://www.uni-potsdam.de/en/headlines-and-featured-stories)
* generate articles based on the real dataset for a wizard-of-oz experiment

## How to run

### Steps

1. copy .env.template to .env and fill in the required variables. Example:
```
# ----------------------------------------------------------------------------------------------------------------
# configure your database connection details here
DB_NAME=python_db
DB_USER=root
DB_PASSWORD=unique_root
DB_HOST=db
MARIADB_ROOT_PASSWORD=unique_root
DB_PORT=3306
# ----------------------------------------------------------------------------------------------------------------
SECRET_KEY=my_secret_string
```
2. build the server
```
docker compose up -d --build
```
3. test if everything is working
```
docker exec -e PATH=/usr/local/bin oneup-django-web-1 python manage.py test --no-input --settings=one_api.settings_prod
```
4. You can use the system and the different views at the following links:
* http://localhost:8000 - landing page on which you can see possible functions
  ![Landing Page](etc/Server_Landingpage.png)

* http://localhost:8000/admin - you have to create one or multiple super users to use the admin interface. Custom
  functions can be seen in the [Custom Admin Functions](#custom-admin-functions)
* http://localhost:5555/flower/dashboard - Flower Dashboard. See all Celery tasks and their results.
* http://localhost:8000/ht/ - health check page - shows if some things work or not
* http://localhost:8100/ - autogenerated Sphinx documentation from docstrings

### Reset every docker step
remove everything and build all from the ground up
```
docker compose down --rmi local -v; docker compose up -d --build --force-recreate --always-recreate-deps
```

### Variables to set to run the gitlab-ci CI / CD pipeline

| variable name            | description                                                                                                                                                                                       |
|--------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| DB_PASSWORD_PROD         | password used to connect to the database                                                                                                                                                          |
| DJANGO_SETTINGS_MODULE   | overwrite default config file                                                                                                                                                                     |
| SECRET_KEY               | django secret key                                                                                                                                                                                 |
| DOCKER_REGISTRY_HOST     | fqdn of private docker registry                                                                                                                                                                   |
| DOCKER_REGISTRY_PASSWORD | password for authentication against private registry                                                                                                                                              |
| DOCKER_REGISTRY_USER     | username for authentication against private registry                                                                                                                                              |
| DOCKER_AUTH_CONFIG       | special formatted json string for the gitlab-runner to use the private registry ([see gitlab docs](https://docs.gitlab.com/ee/ci/docker/using_docker_build.html#option-3-use-docker_auth_config)) |

## Developed Apps

Django provides the ability for multiple applications to be created together within a Django instance. The following
topics are covered by the apps: 1. management and data extraction of the news articles, 2. crop training data from the
UP news portal, 3. training of the prediction model.

* articles: main app to handle articles
    * read out articles and extract data
    * wizard to generate data
    * create OpenAPI
* cropping: crop data from UP news portal
    * save UP related tags
    * save UP-News texts
* one_api: carrier app to config Django
* pr_upload: *deprecated*
* prediction: create prediction model to add topics to the news articles
* wizard: *deprecated*

![Component diagram to visualize the interaction between the apps](etc/REST_API_Components.jpg)

A visualization of the interaction of the components: Prediction annotates existing articles from the articles
component, the training data for the prediction component is loaded from the cropping component, and the cropping
component provides categories for the articles component.

## Custom Admin Functions

### articles.users

* deactivate_user - Deactivate selected users. It isn't a good idea to delete users, because there is a possibility to
  lose important data.
* create_user - Regardless of the user selection, it is possible to generate a number of new users with random numbers (
  between 10,000 and 100,000).
* start_data_integration – Reads all files from the media folder and starts the UP article cropping.
* apply_post_migrations – Executes post_migration functionality that is usually run on startup.

### cropping.articles

* start_up_article_cropping - Starts crawling news articles from the University of Potsdam and extracts articles and
  their associated categories. Usually stops when the CELERY_TASK_TIME_LIMIT - defined in one_api/settings_prod.py - is
  reached. Alternatively, this functionality can be started by visiting `http://localhost:8000/crop/start`.

## Steps to clean data

### Media

* union sub media with main medium, like "Märkische Allgemeine" (main medium) and "Märkische Allgemeine -
  Westhavelländer", "Märkische Allgemeine - Potsdamer Tageszeitung"
