""" All Tasks for articles """
import logging
import os
import datetime
import secrets

from celery import shared_task
from django.utils import timezone

from articles.data_extraction.tasks import extract_information_from_file
from articles.models import Files, Article, Tag, Medium, MediumType, FunctionData
from articles.serializers import union_medium_with_submedia
from cropping.models import Article as CroppingArticle
from one_api.settings_prod import MEDIA_ROOT

logger = logging.getLogger(__name__)


@shared_task
def celery_idle_job(number):
    """ Just a celery task to configure celery right"""
    logger.info("started celery idle task %d", number)
    counting_number = 0
    for i in range(0, 10000):
        counting_number = counting_number + i
    logger.info("idle task %d ist fertig: %d", number, counting_number)


@shared_task
def read_all_media_files():
    """ read all file in the media folder """
    media_dir = os.listdir(MEDIA_ROOT)
    logger.info("read out %i files from media root", len([name for name in media_dir if name.endswith('pdf')]))
    for m_file in list(filter(lambda file_name: file_name.endswith('pdf'), media_dir)):
        if Files.objects.filter(name=m_file).count() == 0:
            file = Files(name=m_file, upload_date=timezone.now())
            file.save()
        elif Files.objects.filter(name=m_file, extraction_complete=False):
            logger.debug("file %s will be re-read", m_file)
            file = Files.objects.get(name=m_file)
            Article.objects.filter(source_file=file).delete()
            extract_information_from_file.delay(file.pk)
        else:
            logger.debug("file %s already exists and was correct extracted in db", m_file)


def generate_article(article_count, upload_date, tag_id, cropping_articles, media):
    """
        Generating article:
        article_count: amount of articles to generate with this call
        upload_date: defines the upload date of the articles that are to be created
        tag_id: id of the tag the article shall be associated with
        cropping_articles: related cropping articles
        media: related media
        First a list of articles with a length equal to article_count is created,
            containing similarily generated fake articles based on wizard_of_oz_faker.pdf file.
            After that, the articles get added to the through tag_id indexed tag.
    """
    file = Files.objects.get(name='wizard_of_oz_faker.pdf')

    objs = []
    for _ in range(article_count):
        if cropping_articles.count > 0:
            gen_title = cropping_articles.articles[secrets.randbelow(cropping_articles.count)].title
        else:
            gen_title = 'Fakeartikel'

        if media.count > 0:
            gen_medium = media.media[secrets.randbelow(media.count)]
        else:
            medium_type, _ = MediumType.objects.update_or_create(name='Zeitung')
            gen_medium, _ = Medium.objects.update_or_create(name=union_medium_with_submedia('fakemedium'), medium_type=medium_type, reach=1000)

        input_data = {
            'date_of_publication': upload_date,
            'text': 'Dies ist ein beispielhafter Beispieltext',
            'pages': 2,
            'wrongly_parsed': False,
        }

        objs.append(Article(medium=gen_medium, source_file=file, name=gen_title, reach=gen_medium.reach, **input_data))

    tagged_obj = []
    tag = Tag.objects.get(id=tag_id)
    for obj in objs:
        tagged_obj.append(Article.tags.through(article=obj, tag=tag))
        obj.save()

    return tagged_obj


def generate_tag(tag_id):
    """
        Generating new fake tag:
        tag_id: suffix of the tag name, distiguishing multiple fake tags
        Creates a unique fake tag with given suffix.
    """
    tag = Tag(name="Fake" + str(tag_id))
    tag.save()


class CroppingArticles:
    """ Bundles the CroppingArticle QuerySet and its count to prevent DB calls in a loop """
    def __init__(self, articles, count):
        self.articles = articles
        self.count = count


class Media:
    """ Bundles the Medium QuerySet and its count to prevent DB calls in a loop """
    def __init__(self, media, count):
        self.media = media
        self.count = count


def create_fake_file_ine():
    """
        Adding fake file if not already exists:
        Checks, if file to relate fake articles to exists and fixes it's non-existance if necessary
    """
    if Files.objects.filter(name__exact='wizard_of_oz_faker.pdf').count() == 0:
        Files.objects.create(name='wizard_of_oz_faker.pdf', upload_date=datetime.datetime.today())


@shared_task
def generate_by_plot():
    """
        Generating fake articles by function graph:
        After checking if file to relate fake articles to exists,
            a fake article creation is started for all existing tags.
    """
    create_fake_file_ine()
    for tag in Tag.objects.exclude(name="account_used").all():
        create_articles_per_tag.delay(tag.id)


@shared_task
def generate_by_id(tag_id):
    """
        Generating fake articles by tag id:
        tag_id: id of tag to generate the fake articles for
        After assuring the existance of file to relate articles to,
            a process to create fake articles for given tag is started.
    """
    create_fake_file_ine()
    create_articles_per_tag.delay(tag_id)


@shared_task
def create_articles_per_tag(tag_id):
    """
        Generating articles based on given tag:
        tag: tag the articles will be created for
        First, all the function data relevant to the given tag will be filtered from db,
            then this data will be skimmed through.
            Every instance of function data saves the amount of articles created for one day.
            For every day the amount of articles, represented by the function is created.
            Finally the generated articles get saved.
    """

    # setup
    cropping_articles = CroppingArticles(CroppingArticle.objects.all(), CroppingArticle.objects.all().count())
    media = Media(Medium.objects.all(), Medium.objects.all().count())
    tagged_objs = []
    start_date = get_start_date()
    tag = Tag.objects.get(id=tag_id)

    # get all the function values for given tag
    qdata = FunctionData.objects.filter(tag=tag)

    # generate articles based on y-axis values per day
    for qdat in qdata:
        tagged_objs += generate_article(qdat.y_data, start_date, tag.id, cropping_articles, media)
        start_date += datetime.timedelta(days=1)

    # insert generated articles into db
    Article.tags.through.objects.bulk_create(tagged_objs)


def get_start_date():
    """Defines and returns the start date for function, plot and article generation"""
    return datetime.date(2021, 12, 1)
